from tests.base import JenkinsTestBase


class JenkinsJobsTestBase(JenkinsTestBase):

    config_xml = """
        <matrix-project>
            <actions/>
            <description>Foo</description>
        </matrix-project>"""


class JenkinsGetJobsTestBase(JenkinsJobsTestBase):

    jobs_in_folder = [
        {'jobs': [
            {'name': 'my_job1', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_folder1', 'url': 'http://...', 'jobs': [{}, {}]},
            {'name': 'my_job2', 'color': 'blue', 'url': 'http://...'},
            {'name': 'job', 'url': 'http://...', 'jobs': [{}]}
        ]},
        # my_folder1 jobs
        {'jobs': [
            {'name': 'my_job3', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_job4', 'color': 'blue', 'url': 'http://...'}
        ]},
        # "job" folder jobs
        {'jobs': [
            {'name': 'my_job', 'color': 'blue', 'url': 'http://...'}
        ]}
    ]

    jobs_in_multiple_folders = [
        {'jobs': [
            {'name': 'my_job1', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_folder1', 'url': 'http://...', 'jobs': [{}, {}, {}]},
            {'name': 'my_job2', 'color': 'blue', 'url': 'http://...'}
        ]},
        # my_folder1 jobs
        {'jobs': [
            {'name': 'my_folder2', 'url': 'http://...', 'jobs': [{}, {}]},
            {'name': 'my_job3', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_job4', 'color': 'blue', 'url': 'http://...'}
        ]},
        # my_folder1/my_folder2 jobs
        {'jobs': [
            {'name': 'my_job1', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_job2', 'color': 'blue', 'url': 'http://...'}
        ]}
    ]

    jobs_in_unsafe_name_folders = [
        {'jobs': [
            {'name': 'my_job1', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_folder1', 'url': 'http://...', 'jobs': [{}, {}]},
            {'name': 'my_job2', 'color': 'blue', 'url': 'http://...'}
        ]},
        # my_folder1 jobs
        {'jobs': [
            {'name': 'my spaced folder', 'url': 'http://...', 'jobs': [{}]},
            {'name': 'my_job3', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_job4', 'color': 'blue', 'url': 'http://...'}
        ]},
        # my_folder1/my\ spaced\ folder jobs
        {'jobs': [
            {'name': 'my job 5', 'color': 'blue', 'url': 'http://...'}
        ]}
    ]

    jobs_in_folder_deep_query = [
        {'jobs': [
            {'name': 'top_folder', 'url': 'http://...', 'jobs': [
                {'name': 'middle_folder', 'url': 'http://...', 'jobs': [
                    {'name': 'bottom_folder', 'url': 'http://...',
                     'jobs': [{}, {}]}
                ]}
            ]}
        ]},
        # top_folder/middle_folder/bottom_folder jobs
        {'jobs': [
            {'name': 'my_job1', 'color': 'blue', 'url': 'http://...'},
            {'name': 'my_job2', 'color': 'blue', 'url': 'http://...'}
        ]}
    ]



import json
from mock import patch

import jenkins
from tests.base import JenkinsTestBase
from tests.helper import build_response_mock


class JenkinsWhoamiTest(JenkinsTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        user_to_return = \
            {u'absoluteUrl': u'https://example.com/jenkins/user/jsmith',
             u'description': None,
             u'fullName': u'John Smith',
             u'id': u'jsmith',
             u'property': [{},
                           {},
                           {},
                           {u'address': u'jsmith@example.com'},
                           {},
                           {},
                           {u'insensitiveSearch': False},
                           {}]}

        jenkins_mock.return_value = json.dumps(user_to_return)

        user = self.j.get_whoami()

        self.assertEqual(user, user_to_return)
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('me/api/json?depth=0'))
        self._check_requests(jenkins_mock.call_args_list)

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_raise_HTTPError(self, session_send_mock):
        session_send_mock.side_effect = iter([
            build_response_mock(404, reason="Not Found"),  # crumb
            build_response_mock(401, reason="Basic Auth Failed"),  # request
        ])

        with self.assertRaises(jenkins.JenkinsException):
            self.j.get_whoami()
        self.assertEqual(
            session_send_mock.call_args_list[1][0][1].url,
            self.make_url('me/api/json?depth=0'))



import json
from mock import patch

import jenkins
from tests.base import JenkinsTestBase


class JenkinsCredentialTestBase(JenkinsTestBase):
    config_xml = """<com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl>
        <scope>GLOBAL</scope>
        <id>Test Credential</id>
        <username>Test-User</username>
        <password>secret123</password>
      </com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl>"""


class JenkinsGetTagTextTest(JenkinsCredentialTestBase):

    def test_simple(self):
        name_to_return = self.j._get_tag_text('id', self.config_xml)
        self.assertEqual('Test Credential', name_to_return)

    def test_failed(self):
        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j._get_tag_text('id', '<xml></xml>')
        self.assertEqual(str(context_manager.exception),
                         'tag[id] is invalidated')

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j._get_tag_text('id', '<xml><id></id></xml>')
        self.assertEqual(str(context_manager.exception),
                         'tag[id] is invalidated')

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j._get_tag_text('id', '<xml><id>   </id></xml>')
        self.assertEqual(str(context_manager.exception),
                         'tag[id] is invalidated')


class JenkinsIsFolderTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_is_folder(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
        ]
        self.assertTrue(self.j.is_folder('Test Folder'))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_is_not_folder(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'org.jenkinsci.plugins.workflow.job.WorkflowJob'}),
        ]
        self.assertFalse(self.j.is_folder('Test Job'))


class JenkinsAssertFolderTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_is_folder(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
        ]
        self.j.assert_folder('Test Folder')

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_is_not_folder(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'org.jenkinsci.plugins.workflow.job.WorkflowJob'}),
        ]
        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.assert_folder('Test Job')
        self.assertEqual(str(context_manager.exception),
                         'job[Test Job] is not a folder')


class JenkinsAssertCredentialTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_credential_missing(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            jenkins.NotFoundException()
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.assert_credential_exists('NonExistent', 'TestFoler')
        self.assertEqual(
            str(context_manager.exception),
            'credential[NonExistent] does not exist'
            ' in the domain[_] of [TestFoler]')
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_credential_exists(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'id': 'ExistingCredential'})
        ]
        self.j.assert_credential_exists('ExistingCredential', 'TestFoler')
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsCredentialExistsTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_credential_missing(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            jenkins.NotFoundException()
        ]

        self.assertEqual(self.j.credential_exists('NonExistent', 'TestFolder'),
                         False)
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_credential_exists(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'id': 'ExistingCredential'})
        ]

        self.assertEqual(self.j.credential_exists('ExistingCredential',
                                                  'TestFolder'),
                         True)
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsGetCredentialInfoTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        credential_info_to_return = {'id': 'ExistingCredential'}
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps(credential_info_to_return)
        ]

        credential_info = self.j.get_credential_info('ExistingCredential', 'TestFolder')

        self.assertEqual(credential_info, credential_info_to_return)
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('job/TestFolder/credentials/store/folder/'
                          'domain/_/credential/ExistingCredential/api/json?depth=0'))
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_nonexistent(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            None,
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.get_credential_info('NonExistent', 'TestFolder')

        self.assertEqual(
            str(context_manager.exception),
            'credential[NonExistent] does not exist '
            'in the domain[_] of [TestFolder]')

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_invalid_json(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            '{invalid_json}'
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.get_credential_info('NonExistent', 'TestFolder')

        self.assertEqual(
            str(context_manager.exception),
            'Could not parse JSON info for credential[NonExistent]'
            ' in the domain[_] of [TestFolder]')


class JenkinsGetCredentialConfigTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_encodes_credential_name(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            None,
        ]
        self.j.get_credential_config(u'Test Credential', u'Test Folder')

        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/domain/'
                          '_/credential/Test%20Credential/config.xml'))
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsCreateCredentialTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            jenkins.NotFoundException(),
            None,
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'id': 'Test Credential'}),
        ]

        self.j.create_credential('Test Folder', self.config_xml)
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/'
                          'domain/_/credential/Test%20Credential/api/json?depth=0'))

        self.assertEqual(
            jenkins_mock.call_args_list[2][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/'
                          'domain/_/createCredentials'))

        self.assertEqual(
            jenkins_mock.call_args_list[4][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/'
                          'domain/_/credential/Test%20Credential/api/json?depth=0'))
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_already_exists(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'id': 'Test Credential'}),
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.create_credential('Test Folder', self.config_xml)
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/'
                          'domain/_/credential/Test%20Credential/api/json?depth=0'))

        self.assertEqual(
            str(context_manager.exception),
            'credential[Test Credential] already exists'
            ' in the domain[_] of [Test Folder]')
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_failed(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            jenkins.NotFoundException(),
            None,
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            None,
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.create_credential('Test Folder', self.config_xml)
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/'
                          'domain/_/credential/Test%20Credential/api/json?depth=0'))
        self.assertEqual(
            jenkins_mock.call_args_list[2][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/'
                          'folder/domain/_/createCredentials'))
        self.assertEqual(
            jenkins_mock.call_args_list[4][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/'
                          'domain/_/credential/Test%20Credential/api/json?depth=0'))
        self.assertEqual(
            str(context_manager.exception),
            'create[Test Credential] failed in the domain[_] of [Test Folder]')
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsDeleteCredentialTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            True,
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            jenkins.NotFoundException(),
        ]

        self.j.delete_credential(u'Test Credential', 'TestFolder')

        self.assertEqual(
            jenkins_mock.call_args_list[0][0][0].url,
            self.make_url('job/TestFolder/credentials/store/folder/domain/'
                          '_/credential/Test%20Credential/config.xml'))
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_failed(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'id': 'ExistingCredential'}),
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'id': 'ExistingCredential'})
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.delete_credential(u'ExistingCredential', 'TestFolder')
        self.assertEqual(
            jenkins_mock.call_args_list[0][0][0].url,
            self.make_url('job/TestFolder/credentials/store/folder/'
                          'domain/_/credential/ExistingCredential/config.xml'))
        self.assertEqual(
            str(context_manager.exception),
            'delete credential[ExistingCredential] from '
            'domain[_] of [TestFolder] failed')
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsReconfigCredentialTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'id': 'Test Credential'}),
            None
        ]

        self.j.reconfig_credential(u'Test Folder', self.config_xml)

        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/domain/'
                          '_/credential/Test%20Credential/api/json?depth=0'))
        self.assertEqual(
            jenkins_mock.call_args_list[2][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/domain/'
                          '_/credential/Test%20Credential/config.xml'))
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsListCredentialConfigTest(JenkinsCredentialTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        credentials_to_return = [{'id': 'Test Credential'}]
        jenkins_mock.side_effect = [
            json.dumps({'_class': 'com.cloudbees.hudson.plugins.folder.Folder'}),
            json.dumps({'credentials': [{'id': 'Test Credential'}]}),
        ]
        credentials = self.j.list_credentials(u'Test Folder')
        self.assertEqual(credentials, credentials_to_return)
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('job/Test%20Folder/credentials/store/folder/domain/'
                          '_/api/json?tree=credentials[id]'))
        self._check_requests(jenkins_mock.call_args_list)



import json
from mock import patch

import jenkins
import requests_mock
from tests.base import JenkinsTestBase
from tests.helper import build_response_mock


class JenkinsNodesTestBase(JenkinsTestBase):

    def setUp(self):
        super(JenkinsNodesTestBase, self).setUp()
        self.node_info = {
            'displayName': 'test node',
            'totalExecutors': 5,
        }
        self.online_node_info = dict(self.node_info)
        self.online_node_info['offline'] = False
        self.offline_node_info = dict(self.node_info)
        self.offline_node_info['offline'] = True


class JenkinsGetNodesTest(JenkinsNodesTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.return_value = json.dumps({
            "computer": [{
                "displayName": "master",
                "offline": False
            }],
            "busyExecutors": 2})
        self.assertEqual(self.j.get_nodes(),
                         [{'name': 'master', 'offline': False}])
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_return_invalid_json(self, jenkins_mock):
        jenkins_mock.side_effect = [
            'Invalid JSON',
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.get_nodes()
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/api/json?depth=0'))
        self.assertEqual(
            str(context_manager.exception),
            'Could not parse JSON info for server[{0}]'.format(
                self.make_url('')))
        self._check_requests(jenkins_mock.call_args_list)

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_raise_BadStatusLine(self, session_send_mock):
        session_send_mock.side_effect = jenkins.BadStatusLine(
            'not a valid status line')
        with self.assertRaises(jenkins.BadHTTPException) as context_manager:
            self.j.get_nodes()
        self.assertEqual(
            str(context_manager.exception),
            'Error communicating with server[{0}]'.format(
                self.make_url('')))

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_raise_HTTPError(self, session_send_mock):
        session_send_mock.side_effect = iter([
            build_response_mock(404, reason="Not Found"),        # crumb
            build_response_mock(499, reason="Unhandled Error"),  # request
        ])

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.get_nodes()
        self.assertEqual(
            session_send_mock.call_args_list[1][0][1].url,
            self.make_url('computer/api/json?depth=0'))
        self.assertEqual(
            str(context_manager.exception),
            'Error communicating with server[{0}]'.format(
                self.make_url('')))


class JenkinsGetNodeInfoTest(JenkinsNodesTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.node_info),
        ]

        self._check_requests(jenkins_mock.call_args_list)
        self.assertEqual(self.j.get_node_info('test node'), self.node_info)
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/test%20node/api/json?depth=0'))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_return_invalid_json(self, jenkins_mock):
        jenkins_mock.side_effect = [
            'Invalid JSON',
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.get_node_info('test_node')

        self._check_requests(jenkins_mock.call_args_list)
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/test_node/api/json?depth=0'))
        self.assertEqual(
            str(context_manager.exception),
            'Could not parse JSON info for node[test_node]')

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_raise_HTTPError(self, session_send_mock):
        session_send_mock.side_effect = iter([
            build_response_mock(404, reason="Not Found"),        # crumb
            build_response_mock(499, reason="Unhandled Error"),  # request
        ])

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.get_node_info('test_node')
        self.assertEqual(
            session_send_mock.call_args_list[1][0][1].url,
            self.make_url('computer/test_node/api/json?depth=0'))
        self.assertEqual(
            str(context_manager.exception),
            'node[test_node] does not exist')


class JenkinsAssertNodeExistsTest(JenkinsNodesTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_node_missing(self, jenkins_mock):
        jenkins_mock.side_effect = [None]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.assert_node_exists('NonExistentNode')

        self._check_requests(jenkins_mock.call_args_list)
        self.assertEqual(
            str(context_manager.exception),
            'node[NonExistentNode] does not exist')

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_node_exists(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps({'name': 'ExistingNode'})
        ]
        self.j.assert_node_exists('ExistingNode')
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsDeleteNodeTest(JenkinsNodesTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.node_info),
            None,
            None,
            None,
        ]

        self.j.delete_node('test node')

        self._check_requests(jenkins_mock.call_args_list)
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('computer/test%20node/doDelete'))
        self.assertFalse(self.j.node_exists('test node'))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_failed(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.node_info),
            None,
            json.dumps(self.node_info),
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.delete_node('test_node')

        self._check_requests(jenkins_mock.call_args_list)
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('computer/test_node/doDelete'))
        self.assertEqual(
            str(context_manager.exception),
            'delete[test_node] failed')


class JenkinsCreateNodeTest(JenkinsNodesTestBase):

    @requests_mock.Mocker()
    def test_simple(self, req_mock):
        req_mock.get(self.make_url(jenkins.CRUMB_URL))
        req_mock.post(self.make_url(jenkins.CREATE_NODE), status_code=200,
                      text='success', headers={'content-length': '7'})
        req_mock.get(
            self.make_url('computer/test%20node/api/json?depth=0'),
            [{'status_code': 404, 'headers': {'content-length': '9'},
              'text': 'NOT FOUND'},
             {'status_code': 200, 'json': {'displayName': 'test%20node'},
              'headers': {'content-length': '20'}}
             ])

        self.j.create_node('test node', exclusive=True)

        actual = req_mock.request_history[2]
        self.assertEqual(actual.url, self.make_url('computer/doCreateItem'))
        self.assertIn('name=test+node', actual.body)
        self.assertTrue(self.j.node_exists('test node'))

    @requests_mock.Mocker()
    def test_urlencode(self, req_mock):
        # resp 0 (don't care about this succeeding)
        req_mock.get(self.make_url(jenkins.CRUMB_URL))
        # resp 2
        req_mock.post(self.make_url(jenkins.CREATE_NODE), status_code=200,
                      text='success', headers={'content-length': '7'})
        # resp 1 & 3
        req_mock.get(
            self.make_url('computer/10.0.0.1%2Btest-node/api/json?depth=0'),
            [{'status_code': 404, 'headers': {'content-length': '9'},
              'text': 'NOT FOUND'},
             {'status_code': 200,
              'json': {'displayName': '10.0.0.1+test-node'},
              'headers': {'content-length': '20'}}
             ])

        params = {
            'port': '22',
            'username': 'juser',
            'credentialsId': '10f3a3c8-be35-327e-b60b-a3e5edb0e45f',
            'host': 'my.jenkins.slave1'
        }
        self.j.create_node(
            # Note the use of a URL-encodable character "+" here.
            '10.0.0.1+test-node',
            nodeDescription='my test slave',
            remoteFS='/home/juser',
            labels='precise',
            exclusive=True,
            launcher=jenkins.LAUNCHER_SSH,
            launcher_params=params)

        actual = req_mock.request_history[2].body
        # As python dicts do not guarantee order so the parameters get
        # re-ordered when it gets processed by requests, verify sections
        # of the URL with self.assertIn() instead of the entire URL
        self.assertIn(u'name=10.0.0.1%2Btest-node', actual)
        self.assertIn(u'type=hudson.slaves.DumbSlave%24DescriptorImpl', actual)
        self.assertIn(u'username%22%3A+%22juser', actual)
        self.assertIn(
            u'stapler-class%22%3A+%22hudson.plugins.sshslaves.SSHLauncher',
            actual)
        self.assertIn(u'host%22%3A+%22my.jenkins.slave1', actual)
        self.assertIn(
            u'credentialsId%22%3A+%2210f3a3c8-be35-327e-b60b-a3e5edb0e45f',
            actual)
        self.assertIn(u'port%22%3A+%2222', actual)
        self.assertIn(u'remoteFS%22%3A+%22%2Fhome%2Fjuser', actual)
        self.assertIn(u'labelString%22%3A+%22precise', actual)

    @requests_mock.Mocker()
    def test_already_exists(self, req_mock):
        req_mock.get(self.make_url(jenkins.CRUMB_URL))
        req_mock.get(
            self.make_url('computer/test_node/api/json?depth=0'),
            status_code=200, json=self.node_info,
            headers={'content-length': '20'}
        )

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.create_node('test_node')
        self.assertEqual(
            str(context_manager.exception),
            'node[test_node] already exists')

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_failed(self, jenkins_mock):
        jenkins_mock.side_effect = [
            None,
            None,
            None,
            None,
        ]

        with self.assertRaises(jenkins.JenkinsException) as context_manager:
            self.j.create_node('test_node')
        self.assertEqual(
            jenkins_mock.call_args_list[1][0][0].url,
            self.make_url('computer/doCreateItem'))
        self.assertEqual(
            str(context_manager.exception),
            'create[test_node] failed')
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsEnableNodeTest(JenkinsNodesTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.offline_node_info),
            None,
        ]

        self.j.enable_node('test node')

        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/test%20node/'
                          'toggleOffline?offlineMessage='))

        jenkins_mock.side_effect = [json.dumps(self.online_node_info)]
        node_info = self.j.get_node_info('test node')
        self.assertEqual(node_info, self.online_node_info)
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_offline_false(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.online_node_info),
            None,
        ]

        self.j.enable_node('test_node')

        # Node was not offline; so enable_node skips toggle
        # Last call to jenkins was to check status
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/test_node/api/json?depth=0'))

        jenkins_mock.side_effect = [json.dumps(self.online_node_info)]
        node_info = self.j.get_node_info('test_node')
        self.assertEqual(node_info, self.online_node_info)
        self._check_requests(jenkins_mock.call_args_list)


class JenkinsDisableNodeTest(JenkinsNodesTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.online_node_info),
            None,
        ]

        self.j.disable_node('test node')

        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/test%20node/'
                          'toggleOffline?offlineMessage='))

        jenkins_mock.side_effect = [json.dumps(self.offline_node_info)]
        node_info = self.j.get_node_info('test node')
        self.assertEqual(node_info, self.offline_node_info)
        self._check_requests(jenkins_mock.call_args_list)

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_offline_true(self, jenkins_mock):
        jenkins_mock.side_effect = [
            json.dumps(self.offline_node_info),
            None,
        ]

        self.j.disable_node('test_node')

        # Node was already offline; so disable_node skips toggle
        # Last call to jenkins was to check status
        self.assertEqual(
            jenkins_mock.call_args[0][0].url,
            self.make_url('computer/test_node/api/json?depth=0'))

        jenkins_mock.side_effect = [json.dumps(self.offline_node_info)]
        node_info = self.j.get_node_info('test_node')
        self.assertEqual(node_info, self.offline_node_info)
        self._check_requests(jenkins_mock.call_args_list)



import json
from mock import patch

import jenkins
from tests.jobs.base import JenkinsGetJobsTestBase


class JenkinsGetAllJobsTest(JenkinsGetJobsTestBase):

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_simple(self, jenkins_mock):
        jenkins_mock.side_effect = map(json.dumps, self.jobs_in_folder)

        jobs_info = self.j.get_all_jobs()

        expected_fullnames = [
            u"my_job1", u"my_folder1", u"my_job2", u"job",
            u"my_folder1/my_job3", u"my_folder1/my_job4", u"job/my_job"
        ]
        self.assertEqual(len(expected_fullnames), len(jobs_info))
        got_fullnames = [job[u"fullname"] for job in jobs_info]
        self.assertEqual(expected_fullnames, got_fullnames)

        expected_request_urls = [
            self.make_url('api/json'),
            self.make_url('job/my_folder1/api/json'),
            self.make_url('job/job/api/json')
        ]
        self.assertEqual(expected_request_urls,
                         self.got_request_urls(jenkins_mock))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_multi_level(self, jenkins_mock):
        jenkins_mock.side_effect = map(
            json.dumps, self.jobs_in_multiple_folders)

        jobs_info = self.j.get_all_jobs()

        expected_fullnames = [
            u"my_job1", u"my_folder1", u"my_job2", u"my_folder1/my_folder2",
            u"my_folder1/my_job3", u"my_folder1/my_job4",
            u"my_folder1/my_folder2/my_job1", u"my_folder1/my_folder2/my_job2"
        ]
        self.assertEqual(len(expected_fullnames), len(jobs_info))
        got_fullnames = [job[u"fullname"] for job in jobs_info]
        self.assertEqual(expected_fullnames, got_fullnames)
        # multiple jobs with same name
        self.assertEqual(2, len([True
                                 for job in jobs_info
                                 if job['name'] == u"my_job1"]))

        expected_request_urls = [
            self.make_url('api/json'),
            self.make_url('job/my_folder1/api/json'),
            self.make_url('job/my_folder1/job/my_folder2/api/json')
        ]
        self.assertEqual(expected_request_urls,
                         self.got_request_urls(jenkins_mock))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_folders_depth(self, jenkins_mock):
        jenkins_mock.side_effect = map(
            json.dumps, self.jobs_in_multiple_folders)

        jobs_info = self.j.get_all_jobs(folder_depth=1)

        expected_fullnames = [
            u"my_job1", u"my_folder1", u"my_job2", u"my_folder1/my_folder2",
            u"my_folder1/my_job3", u"my_folder1/my_job4"
        ]
        self.assertEqual(len(expected_fullnames), len(jobs_info))
        got_fullnames = [job[u"fullname"] for job in jobs_info]
        self.assertEqual(expected_fullnames, got_fullnames)

        expected_request_urls = [
            self.make_url('api/json'),
            self.make_url('job/my_folder1/api/json')
        ]
        self.assertEqual(expected_request_urls,
                         self.got_request_urls(jenkins_mock))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_unsafe_chars(self, jenkins_mock):
        jenkins_mock.side_effect = map(
            json.dumps, self.jobs_in_unsafe_name_folders)

        jobs_info = self.j.get_all_jobs()

        expected_fullnames = [
            u"my_job1", u"my_folder1", u"my_job2", u"my_folder1/my spaced folder",
            u"my_folder1/my_job3", u"my_folder1/my_job4",
            u"my_folder1/my spaced folder/my job 5"
        ]
        self.assertEqual(len(expected_fullnames), len(jobs_info))
        got_fullnames = [job[u"fullname"] for job in jobs_info]
        self.assertEqual(expected_fullnames, got_fullnames)

        expected_request_urls = [
            self.make_url('api/json'),
            self.make_url('job/my_folder1/api/json'),
            self.make_url('job/my_folder1/job/my%20spaced%20folder/api/json')
        ]
        self.assertEqual(expected_request_urls,
                         self.got_request_urls(jenkins_mock))

    @patch.object(jenkins.Jenkins, 'jenkins_open')
    def test_deep_query(self, jenkins_mock):
        jenkins_mock.side_effect = map(
            json.dumps, self.jobs_in_folder_deep_query)

        jobs_info = self.j.get_all_jobs()

        expected_fullnames = [
            u"top_folder",
            u"top_folder/middle_folder",
            u"top_folder/middle_folder/bottom_folder",
            u"top_folder/middle_folder/bottom_folder/my_job1",
            u"top_folder/middle_folder/bottom_folder/my_job2"
        ]
        self.assertEqual(len(expected_fullnames), len(jobs_info))
        got_fullnames = [job[u"fullname"] for job in jobs_info]
        self.assertEqual(expected_fullnames, got_fullnames)

        expected_request_urls = [
            self.make_url('api/json'),
            self.make_url('job/top_folder/job/middle_folder/job/bottom_folder/api/json')
        ]
        self.assertEqual(expected_request_urls,
                         self.got_request_urls(jenkins_mock))



# -*- coding: utf-8 -*-
from mock import patch

import jenkins
from six.moves.urllib.parse import quote
from tests.helper import build_response_mock
from tests.jobs.base import JenkinsJobsTestBase


class JenkinsBuildJobTest(JenkinsJobsTestBase):

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_simple(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(
            302, {}, headers={'Location': self.make_url('/queue/item/25/')})

        queue_id = self.j.build_job(u'Test Jøb')

        self.assertEqual(session_send_mock.call_args[0][1].url,
                         self.make_url(quote(u'job/Test Jøb/build'.encode('utf8'))))
        self.assertEqual(queue_id, 25)

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_assert_no_location(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(302, {})

        with self.assertRaises(jenkins.EmptyResponseException) as context_mgr:
            self.j.build_job(u'Test Job')

        self.assertEqual(
            str(context_mgr.exception),
            "Header 'Location' not found in response from server[{0}]".format(
                self.make_url('')))

    @patch.object(jenkins.Jenkins, 'maybe_add_crumb')
    @patch('jenkins.requests.Session.send', autospec=True)
    def test_simple_no_content_lenght(self, session_send_mock,
                                      maybe_add_crumb_mock):
        maybe_add_crumb_mock.return_value = None
        session_send_mock.return_value = build_response_mock(
            201, None, add_content_length=False,
            headers={'Location': self.make_url('/queue/item/25/')})

        queue_id = self.j.build_job(u'Test Job')

        self.assertEqual(session_send_mock.call_args[0][1].url,
                         self.make_url('job/Test%20Job/build'))
        self.assertEqual(queue_id, 25)

    @patch.object(jenkins.Jenkins, 'maybe_add_crumb')
    @patch('jenkins.requests.Session.send', autospec=True)
    def test_assert_no_content_lenght_no_location(self, session_send_mock,
                                                  maybe_add_crumb_mock):
        maybe_add_crumb_mock.return_value = None
        session_send_mock.return_value = build_response_mock(
            201, None, add_content_length=False)

        with self.assertRaises(jenkins.EmptyResponseException) as context_mgr:
            self.j.build_job(u'Test Job')

        self.assertEqual(
            str(context_mgr.exception),
            'Error communicating with server[{0}]: empty response'.format(
                self.make_url('')))

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_in_folder(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(
            302, {}, headers={'Location': self.make_url('/queue/item/25/')})

        queue_id = self.j.build_job(u'a Folder/Test Job')

        self.assertEqual(session_send_mock.call_args[0][1].url,
                         self.make_url('job/a%20Folder/job/Test%20Job/build'))
        self.assertEqual(queue_id, 25)

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_with_token(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(
            302, {}, headers={'Location': self.make_url('/queue/item/25/')})

        queue_id = self.j.build_job(u'TestJob', token='some_token')

        self.assertEqual(session_send_mock.call_args[0][1].url,
                         self.make_url('job/TestJob/build?token=some_token'))
        self.assertEqual(queue_id, 25)

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_in_folder_with_token(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(
            302, {}, headers={'Location': self.make_url('/queue/item/25/')})

        queue_id = self.j.build_job(u'a Folder/TestJob', token='some_token')

        self.assertEqual(session_send_mock.call_args[0][1].url,
                         self.make_url('job/a%20Folder/job/TestJob/build?token=some_token'))
        self.assertEqual(queue_id, 25)

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_with_parameters_and_token(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(
            302, {}, headers={'Location': self.make_url('/queue/item/25/')})

        queue_id = self.j.build_job(
            u'TestJob',
            parameters={'when': 'now', 'why': 'because I felt like it'},
            token='some_token')

        self.assertTrue('token=some_token' in session_send_mock.call_args[0][1].url)
        self.assertTrue('when=now' in session_send_mock.call_args[0][1].url)
        self.assertTrue('why=because+I+felt+like+it' in session_send_mock.call_args[0][1].url)
        self.assertEqual(queue_id, 25)



import jenkins
from tests.base import JenkinsTestBase

# Vars in the jenkins module scope that do not need validating
VAR_WHITELIST = ['__doc__',
                 '__file__',
                 '__name__',
                 '__package__',
                 'CRUMB_URL',
                 'DEFAULT_HEADERS',
                 'EMPTY_CONFIG_XML',
                 'EMPTY_FOLDER_XML',
                 'EMPTY_PROMO_CONFIG_XML',
                 'EMPTY_VIEW_CONFIG_XML',
                 'INFO',
                 'LAUNCHER_SSH',
                 'LAUNCHER_COMMAND',
                 'LAUNCHER_JNLP',
                 'LAUNCHER_WINDOWS_SERVICE',
                 'NODE_TYPE',
                 'PROMO_RECONFIG_XML',
                 'RECONFIG_XML']


class JenkinsRestTest(JenkinsTestBase):

    # If there is no filter (depth or tree) we will get an exception
    # on some Jenkins instances
    def test_url_has_filter(self):
        for var in dir(jenkins):
            if var in VAR_WHITELIST:
                continue
            # Misses unicode on 2.x
            val = getattr(jenkins, var)
            if isinstance(val, str):
                # If we end the path in api/json and don't have depth or tree encoded, fail
                self.assertEqual(val.endswith('api/json'), False,
                                 "URLS that end in 'api/json' must be called with depth or tree:" +
                                 "var: [{}] val: [{}]".format(var, val))



import sys

import mock
from testscenarios import TestWithScenarios

import jenkins

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class JenkinsTestBase(TestWithScenarios, unittest.TestCase):

    crumb_data = {
        "crumb": "dab177f483b3dd93483ef6716d8e792d",
        "crumbRequestField": ".crumb",
    }

    scenarios = [
        ('base_url1', dict(base_url='http://example.com')),
        ('base_url2', dict(base_url='http://example.com/jenkins'))
    ]

    def setUp(self):
        super(JenkinsTestBase, self).setUp()

        self.request_kerberos_module_patcher = mock.patch(
            'jenkins.requests_kerberos', None)
        self.request_kerberos_module_patcher.start()

        self.j = jenkins.Jenkins(self.base_url, 'test', 'test')

    def tearDown(self):

        self.request_kerberos_module_patcher.stop()

    def make_url(self, path):
        return u'{0}/{1}'.format(self.base_url, path)

    def _check_requests(self, requests):

        for req in requests:
            req[0][0].prepare()

    def got_request_urls(self, mock):
        return [
            call[0][0].url.split('?')[0]
            for call in mock.call_args_list
        ]



from mock import patch

import jenkins
from tests.base import JenkinsTestBase
from tests.helper import build_response_mock


class JenkinsVersionTest(JenkinsTestBase):

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_some_version(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(
            200, headers={'X-Jenkins': 'Version42', 'Content-Length': 0})
        self.assertEqual(self.j.get_version(), 'Version42')

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_raise_HTTPError(self, session_send_mock):
        session_send_mock.side_effect = iter([
            build_response_mock(404, reason="Not Found"),        # crumb
            build_response_mock(499, reason="Unhandled Error"),  # request
        ])

        with self.assertRaises(jenkins.BadHTTPException) as context_manager:
            self.j.get_version()
        self.assertEqual(
            str(context_manager.exception),
            'Error communicating with server[{0}/]'.format(self.base_url))

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_raise_BadStatusLine(self, session_send_mock):
        session_send_mock.side_effect = jenkins.BadStatusLine('not a valid status line')
        with self.assertRaises(jenkins.BadHTTPException) as context_manager:
            self.j.get_version()
        self.assertEqual(
            str(context_manager.exception),
            'Error communicating with server[{0}/]'.format(self.base_url))

    @patch('jenkins.requests.Session.send', autospec=True)
    def test_return_empty_response(self, session_send_mock):
        session_send_mock.return_value = build_response_mock(0)
        with self.assertRaises(jenkins.EmptyResponseException) as context_manager:
            self.j.get_version()
        self.assertEqual(
            str(context_manager.exception),
            'Error communicating with server[{0}/]:'
            ' empty response'.format(self.base_url))
